//=============================================================================
//  MuseScore
//  Linux Music Score Editor
//  $Id: alsa.h,v 1.8 2006/03/03 21:47:11 wschweer Exp $
//
//  Copyright (C) 2002-2008 Werner Schweer and others
//
//  AlsaDriver based on code from Fons Adriaensen (clalsadr.cc)
//  Copyright (C) 2003 Fons Adriaensen
//  partly based on original work from Paul Davis
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

#ifndef __ALSA_H__
#define __ALSA_H

#include "driver.h"
#include "mididriver.h"
#include "config.h"
#include "fluid.h"

#define ALSA_PCM_NEW_HW_PARAMS_API
#define ALSA_PCM_NEW_SW_PARAMS_API

class MidiDriver;

#include <alsa/asoundlib.h>

//---------------------------------------------------------
//   AlsaDriver
//---------------------------------------------------------

class AlsaDriver {
      QString _name;
      typedef char* (*play_function)(const float*, char*, int, int);
      typedef char* (*clear_function)(char*, int, int);

      enum { MAXPFD = 8, MAXPLAY = 4 };

      int setHwpar(snd_pcm_t* handle, snd_pcm_hw_params_t* hwpar);
      int setSwpar(snd_pcm_t* handle, snd_pcm_sw_params_t* swpar);
      int recover();

      unsigned int           _rate;
      snd_pcm_uframes_t      _frsize;
      unsigned int           _nfrags;
      snd_pcm_format_t       _play_format;
      snd_pcm_access_t       _play_access;
      snd_pcm_t*             _play_handle;
      snd_pcm_hw_params_t*   _play_hwpar;
      snd_pcm_sw_params_t*   _play_swpar;
      unsigned int           _play_nchan;
      int                    _play_npfd;
      struct pollfd          _pfd [MAXPFD];
      snd_pcm_uframes_t      _capt_offs;
      snd_pcm_uframes_t      _play_offs;
      int                    _play_step;
      char*                  _play_ptr [MAXPLAY];
      int                    _stat;
      int                    _pcnt;
      bool                   _xrun;
      clear_function         _clear_func;
      play_function          _play_func;

      static char* clear_32le(char* dst, int step, int nfrm);
      static char* clear_24le(char* dst, int step, int nfrm);
      static char* clear_16le(char* dst, int step, int nfrm);
      static char* play_32le(const float* src, char* dst, int step, int nfrm);
      static char* play_24le(const float* src, char* dst, int step, int nfrm);
      static char* play_16le(const float* src, char* dst, int step, int nfrm);
      int playInit(snd_pcm_uframes_t len);
      snd_pcm_sframes_t pcmWait();
      snd_pcm_t* playHandle() const   { return _play_handle; }
      void clearChan(int chan, snd_pcm_uframes_t len) {
            _play_ptr[chan] = _clear_func(_play_ptr[chan], _play_step, len);
            }

    public:
      AlsaDriver(QString, unsigned, snd_pcm_uframes_t, unsigned);
      ~AlsaDriver();
      bool init();
      void printinfo();
      bool pcmStart();
      int pcmStop();
      snd_pcm_uframes_t fsize() const { return _frsize;      }
      unsigned int sampleRate() const { return _rate; }
      void write(int n, float* l, float* r);
      };

//---------------------------------------------------------
//   AlsaAudio
//---------------------------------------------------------

class AlsaAudio : public Driver {
      pthread_t thread;
      AlsaDriver* alsa;
      volatile int runAlsa;
      int state;
      bool seekflag;
      double startTime;
      ISynth* synth;

      Port midiInPort;
      MidiDriver* midiDriver;

      void registerClient();

   public:
      AlsaAudio(Seq*);
      virtual ~AlsaAudio();
      virtual bool init();
      void* registerPort(const char* name);
      void unregisterPort(void* p);
      virtual QList<QString> inputPorts();
      virtual bool start();
      virtual bool stop();
      void connect(void*, void*) {}
      void disconnect(void* src, void* dst);
      float* getLBuffer(long n);
      float* getRBuffer(long n);
      virtual bool isRealtime() const   { return false; }
      virtual void startTransport();
      virtual void stopTransport();
      virtual int getState();
      virtual int sampleRate() const;
      void alsaLoop();
      void write(int n, void* l, void* r);

      virtual void putEvent(const MidiOutEvent&);
      virtual void process(int, float*, float*, int);
      virtual void midiRead();
      virtual const MidiPatch* getPatchInfo(bool onlyDrums, const MidiPatch*);
      };

#endif

